#ifndef ETHERNET_C
#define ETHERNET_C

#include "ethernet.h"

//write to ethernet controller//////////////////////////////////////////////////
void iow(unsigned int reg, unsigned int data)
{
  IOWR(DM9000A_BASE,IO_addr,reg);
  usleep(STD_DELAY);
  IOWR(DM9000A_BASE,IO_data,data);
}

//read from ethernet controller/////////////////////////////////////////////////
unsigned int ior(unsigned int reg)
{
  IOWR(DM9000A_BASE,IO_addr,reg);
  usleep(STD_DELAY);
  return IORD(DM9000A_BASE,IO_data);
}

//write to phy//////////////////////////////////////////////////////////////////
void phy_write(unsigned int reg, unsigned int value)
{
  //put phy register address into EPAR register
  iow(0x0C, reg | 0x40);
  
  //put phy data to write into EPDR registers
  iow(0x0E, ((value >> 8) & 0xFF));
  iow(0x0D, value & 0xFF);

  // issue PHY write command
  iow(0x0B, 0x8);                     //clear PHY command
  IOWR(DM9000A_BASE, IO_data, 0x0A);  //issue PHY write command
  usleep(STD_DELAY);
  IOWR(DM9000A_BASE, IO_data, 0x08);  //clear PHY command again
  usleep(50);                         // wait 1~30 us (>20 us) for PHY write completion
}

//init ethernet/////////////////////////////////////////////////////////////////
unsigned int DM9000_init (void)
{
  unsigned int  i;

  //turn on the internal PHY
  iow(0x1E, 0x01);  //set GPIO to normal settings
  iow(0x1F, 0x00);  //activate internal PHY
  msleep(5);        //wait 5 ms for PHY to power up

  // software-RESET NIC
  iow(NCR, 0x03);   //reset and turn on MAC loopback
  usleep(20);       //wait 20 us for NIC to reset
  iow(NCR, 0x00);   //set NCR back to normal
  iow(NCR, 0x03);   //repeat
  usleep(20);
  iow(NCR, 0x00);
  
  //turn the PHY back off and then on again
  iow(0x1F, 0x01);  //turn off PHY
  iow(0x1F, 0x00);  //turn on PHY
  msleep(10);       //wait 10 ms for PHY to power up again
  
  //set PHY operation mode
  phy_write(0,PHY_reset);   //reset PHY registers to default status
  usleep(50);               //wait 50 us for PHY registers to reset
  phy_write(16, 0x404);     //turn off low power mode
  phy_write(4, PHY_txab);   //set PHY TX advertised ability: ALL + Flow_control
  phy_write(0, 0x1200);     //enable link speed auto-negotiation
  msleep(5);                //wait 5 ms for PHY to negotiate link speed

  //set NIC MAC address
  for (i = 0; i < 6; i++) 
  iow(16 + i, ether_addr[i]);
  
  //read back MAC address for verification
  vga_write_output(CHECKMARK);
  vga_output("MAC address readback ");
  vga_hex_output(ior(16),1);
  for (i = 1; i < 6; i++) 
    vga_hex_output(ior(16 + i),0);
  vga_return_output();
  
  //set internal clock to 100 MHz - does not work
  //iow(0x2E, 0x80);

  //Clear any pending interrupt
  iow(ISR, 0x3F);     //clear the interrupt status register
  iow(NSR, 0x2C);     //clear the TXD status

  //program the operating registers
  iow(NCR,  NCR_set);   //enable chip and set loopback mdoe to normal
  iow(0x08, BPTR_set);  //set RX back pressure threshold (generates Jam pattern if SRAM is almost full)
  iow(0x09, FCTR_set);  //set flow control register
  iow(0x0A, RTFCR_set); //enable RX/TX flow control
  iow(0x0F, 0x00);      //clear the all event
  iow(0x2D, 0x80);      //switch LED to mode 1
  
  //enable interrupts
  iow(IMR, INTR_set);   //pointer auto return, receive, link state changed

  //enable receiver and GO
  iow(RCR , RCR_set | RX_ENABLE | PASS_MULTICAST | PROMISC);  //enable reception of broadcast, multicast, and promiscuous mode
 
  return  ((ior(NSR)&0xC0)|(ior(NCR)&0x08));  //return link state information
}

//transmit one packet///////////////////////////////////////////////////////////
unsigned int TransmitPacket(unsigned char *data_ptr,unsigned int tx_len)
{
  //don't ever transmit anything
  return 0;
}

//receive one packet////////////////////////////////////////////////////////////
unsigned int ReceivePacket (unsigned char *data_ptr,unsigned int *rx_len)
{
  unsigned char rx_READY,GoodPacket;
  unsigned int  Tmp, RxStatus, i;
  
  RxStatus = rx_len[0] = 0;
  GoodPacket=FALSE;

   /* mask NIC interrupts IMR: PAR only */
  iow(IMR, PAR_set);
  
  /* dummy read a byte from MRCMDX REG. F0H */
  rx_READY = ior(MRCMDX);
  
  /* got most updated byte: rx_READY */
  rx_READY = IORD(DM9000A_BASE,IO_data)&0x03;
  usleep(STD_DELAY);
  
  /* check if (rx_READY == 0x01): Received Packet READY? */
  if (rx_READY == DM9000_PKT_READY)
  {
    /* got RX_Status & RX_Length from RX SRAM */
    IOWR(DM9000A_BASE, IO_addr, MRCMD); /* set MRCMD REG. F2H RX I/O port ready */
    usleep(STD_DELAY);
    RxStatus = IORD(DM9000A_BASE,IO_data);
    usleep(STD_DELAY);
    rx_len[0] = IORD(DM9000A_BASE,IO_data);

    /* Check this packet_status GOOD or BAD? */
    if ( !(RxStatus & 0xBF00) && (rx_len[0] < MAX_PACKET_SIZE) )
    {
      /* read 1 received packet from RX SRAM into RX buffer */
      for (i = 0; i < rx_len[0]; i += 2)
      {
        usleep(STD_DELAY);
        Tmp = IORD(DM9000A_BASE, IO_data);
        data_ptr[i] = Tmp&0xFF;
        data_ptr[i+1] = (Tmp>>8)&0xFF;
      }
      GoodPacket=TRUE;
    } /* end if (GoodPacket) */
    else
    {
      /* this packet is bad, dump it from RX SRAM */
      for (i = 0; i < rx_len[0]; i += 2)
      {
        usleep(STD_DELAY);
        Tmp = IORD(DM9000A_BASE, IO_data);        
      }
      rx_len[0] = 0;
    } /* end if (!GoodPacket) */
    iow(ISR, 0x01);  //added by alex
  } /* end if (rx_READY == DM9000_PKT_READY) ok */
  else if(rx_READY) /* status check first byte: rx_READY Bit[1:0] must be "00"b or "01"b */
  {
    /* software-RESET NIC */
    iow(NCR, 0x03);   /* NCR REG. 00 RST Bit [0] = 1 reset on, and LBK Bit [2:1] = 01b MAC loopback on */
    usleep(20);       /* wait > 10us for a software-RESET ok */
    iow(NCR, 0x00);   /* normalize */
    iow(NCR, 0x03);
    usleep(20);
    iow(NCR, 0x00);   
     
    /* program operating registers~ */
    iow(NCR,  NCR_set);   /* NCR REG. 00 enable the chip functions (and disable this MAC loopback mode back to normal) */
    iow(0x08, BPTR_set);  /* BPTR  REG.08  (if necessary) RX Back Pressure Threshold in Half duplex moe only: High Water 3KB, 600 us */
    iow(0x09, FCTR_set);  /* FCTR  REG.09  (if necessary) Flow Control Threshold setting High/ Low Water Overflow 5KB/ 10KB */
    iow(0x0A, RTFCR_set); /* RTFCR REG.0AH (if necessary) RX/TX Flow Control Register enable TXPEN, BKPM (TX_Half), FLCE (RX) */
    iow(0x0F, 0x00);      //clear all wakeup events
    iow(0x2D, 0x80);      //switch led to mode 1
    /* set other registers depending on applications */
    //iow(ETXCSR, ETXCSR_set); /* Early Transmit 75% */
    /* enable interrupts to activate DM9000 ~on */
    iow(IMR, INTR_set);   //enable pointer auto return, receive packet interrupt
    /* enable RX broadcast, multicast, promiscuous*/
    iow(RCR , RCR_set | RX_ENABLE | PASS_MULTICAST | PROMISC);  
  } /* end NIC H/W system Data-Bus error */
  
  return GoodPacket ? DMFE_SUCCESS : DMFE_FAIL;
}

//handle ethernet interrupts from DM9000A///////////////////////////////////////
void handle_eth_ints()
{
  alt_irq_disable(PROCESS_TIMER_IRQ);
  unsigned int status;
  int rx_len;
  unsigned char data_rx[MAX_PACKET_SIZE];
  status = ior(ISR); //read interrupt status register
  iow(IMR, 0x80);  //kill all interrupts 
  if(status & 0x20) //if link state changed
  {
    //display timestamp and link status
    vga_dec_output(hours);
    vga_write_output(':');
    vga_dec_output(minutes);
    vga_write_output(':');
    vga_dec_output(seconds);
    vga_output(" Link ");
    eth_status = ((ior(NSR)&0xC0)|(ior(NCR)&0x08)); //update link state
    if(!(eth_status & 0x40))
    {
      vga_output("went down\n\n");
    }
    else
    {
      vga_output("went up "); 
      if(eth_status&0x80)
      {
        vga_output("10M HDX\n\n");
      }
      else
      {
        vga_output("100M ");
        if(eth_status&0x08)
        {
          vga_output("FDX\n\n");
        }
        else
        {
          vga_output("HDX\n\n");
        }
      }
    }
    iow(ISR, 0x20); //clear interrupt
  }
  if(status & 0x80) //if pointer auto-returned
  {
    iow(ISR, 0x80); //clear interrupt 
  }
  if(status & 0x01) //if frame received
  {
    //use a while loop in case more packets arrive while we are
    //processing the current one
    while(!ReceivePacket(data_rx, &rx_len))
    {
      //increment packets rxd counter
      pkts_rxd++;
      
      //if the device passes the filter, process it
      if(ApplyFilters(data_rx, &rx_len)==FILTER_PASS)
        parse_eth_frame(data_rx, rx_len);
    }
    iow(ISR, 0x01);   //clear interrupt
    usleep(5);
  }
  iow(IMR,INTR_set); //restore interrupts
  alt_irq_enable(PROCESS_TIMER_IRQ);
}   

#endif
