#ifndef LOG_C
#define LOG_C

#include "log.h"

//global variables//////////////////////////////////////////////////////////////

//this is the flash device
alt_flash_cfi_dev fDev;

//this is the current log write offset
int logOffset = 20;

//this is the next sector where log data will be written
int logSector = 1;

//this is the offset where we start reading
unsigned long int readOffset = 19;

//this flag denotes that we are showing the log.  When it is set,
//entering the quit command will stop showing the log.  Any other command
//input will display the next page of log data.
unsigned short int showingLog=0;

//The CFI flash is divided into sectors 65 kB long.  Since writing to a sector
//will erase other data in that sector, we must get a whole page of data, then
//write it to flash all at once if we are logging.  This buffer stores log 
//entries until there is enough data to write it all to flash.
//NOTE: it may be possible to avoid this behavior by replacing the altera flash
//drivers with better ones, but I don't have time right now.
unsigned char logBuffer[65535] = "__ESNIFF_LOG_BEGIN__\n";

//initialize flash memory system////////////////////////////////////////////////
void log_init(unsigned char* data)
{
  //initialize the flash device
  //this creates a flash device structure called cfi_flash_0
  ALTERA_AVALON_CFI_FLASH_INSTANCE( CFI_FLASH_0, cfi_flash_0);
  ALTERA_AVALON_CFI_FLASH_INIT( CFI_FLASH_0, cfi_flash_0 );
  
  //save the flash device struct to a global
  fDev = cfi_flash_0; 
}

//flush the log buffer//////////////////////////////////////////////////////////
//note: this takes about half a second. during this time a lot of packets may be
//dropped.  Is there a better way to do this?  I don't know what it is.
//I think flash writes are time-critical so they can't be interrupted,
//or otherwise I would do this in an application context and let the ethernet
//interrupts fire over the top of it.
void log_buffer_flush()
{
  //if the logOffset is not 65534 (i.e. the last byte in this sector)
  //we must append a '/0' character to the buffer to signify the end of the log
  //otherwise, we must write a '/0' to the start of the next sector
  if(logOffset==65534)
  {
    alt_flash_cfi_write(&fDev.dev,logSector*0x10000,logBuffer,65535);
    alt_flash_cfi_write(&fDev.dev,(logSector+1)*0x10000,"",1);
  }
  else
  {
    logBuffer[++logOffset]='\0';
    alt_flash_cfi_write(&fDev.dev,logSector*0x10000,logBuffer,65535);
  }
} 

//log a null-terminated string//////////////////////////////////////////////////
void log_data(unsigned char* data)
{
  //control logging here - if we're not logging, return
  if(logging==0)
    return;
  
  while(*data != '\0')
  { 
    //if flash is full, stop writing to it.
    if (logSector == 71)
    {
      vga_output("Flash memory is full.  Logging Deactivated!!!\n\n");
      logging=0;
      return; 
    }
    //copy data into buffer
    logBuffer[logOffset]=*data;   
    
    //if we have a full buffer, write it to flash
    if(logOffset == 65534)
    {
      log_buffer_flush();
      logOffset=0;
      logSector++;
    }
    //else increment the offset
    else
    {
      logOffset++;
    }
    
    //increment the data pointer
    data++;
  }
}

//log a null-terminated string//////////////////////////////////////////////////
void log_char(unsigned char data)
{
  //control logging here - if we're not logging, return
  if(logging==0)
    return;
     
  //if flash is full, stop writing to it.
  if (logSector == 71)
  {
    vga_output("Flash memory is full.  Logging Deactivated!!!\n\n");
    logging=0;
    return; 
  }
  
  //copy data into buffer
  logBuffer[logOffset]=data;   
  
  //if we have a full buffer, write it to flash
  if(logOffset == 65534)
  {
    log_buffer_flush();
    logOffset=0;
    logSector++;
  }
  //else increment the offset
  else
  {
    logOffset++;
  }
}

//log a hex value///////////////////////////////////////////////////////////////
void log_hex_data(unsigned short int data, int prefix)
{
  if(prefix==1)
    log_data("0x");
  unsigned char tmp[3] = {(data & 0xF0),(data & 0x0F),'\0'};
  if((tmp[0]>>4) < 10)
  {
    tmp[0]=(tmp[0]>>4)|0x30;
  }
  else
  {
    tmp[0]=(tmp[0]>>4)+0x37;
  }
  if(tmp[1] < 10)
  {
    tmp[1]=tmp[1]|0x30;
  }
  else
  {
    tmp[1]=tmp[1]+0x37;
  }
  log_data(tmp);
}

//log a decimal value///////////////////////////////////////////////////////////
void log_dec_data(unsigned int data)
{
   int trail=0, j=0;
   if((j=data/10000) || trail)
   {
      log_char((j)|0x30);
      trail = 1;
      data=data%10000;
   }
   if((j=data/1000) || trail)
   {
      log_char((j)|0x30);
      trail = 1;
      data=data%1000;
   }
   if((j=data/100) || trail)
   {
      log_char((j)|0x30);
      trail = 1;
      data=data%100;
   }
   if((j=data/10) || trail)
   {
      log_char((j)|0x30);
      trail = 1;
      data=data%10;
   }
   log_char(data|0x30);
}

//clear the log/////////////////////////////////////////////////////////////////
void log_clear()
{
  //don't actually clear the log, just erase the __ESNIFF_LOG_BEGIN__ string
  //so E-Sniff won't recognize the log data. (i.e. write some crap to sector 1)
  vga_output("Clearing packet log...");
  alt_flash_cfi_write(&fDev.dev,0x10000,"NO LOG",6);
}

//show the log//////////////////////////////////////////////////////////////////
void log_show()
{  
  //make sure there is a log
  unsigned char tmp[20];
  alt_flash_cfi_read(&fDev.dev,0x10000,tmp,20);
  if(!strcmpbeg(tmp,"__ESNIFF_LOG_BEGIN__"))
  {
     vga_output("No log to display.\n\n");
     return;
  }
 
  //start by disabling logging & capture
  logging = 0;
  capturing = 0;
  alt_irq_disable(DM9000A_IRQ);
  
  //display next page of the log
  int linecount=0;
  while(readOffset++)
  {    
    //read in the next byte
    unsigned char c[2];
    alt_flash_cfi_read(&fDev.dev,0x10000+readOffset,&c,1);
    c[1]='\0';
    
    //if the character is '\0', quit
    if(*c=='\0')
    {
      vga_output("END OF LOG\n\n");
      showingLog=0;
      readOffset=19;
      return; 
    }
    
    //display it
    vga_output(c);
    
    //if the character is \n, increment the line count
    if(*c=='\n')
      linecount++;
      
    //if the line count is 51, we set a flag for the keyboard handler
    //if the keyboard interrupt is triggered while the flag = 1, the
    //keyboard interrupt will vector back to here to show the next screen of
    //log data.  We can just set the flag and quit this way.
    //typing quit will clear the flag and reset the indices used to read log data.
    if(linecount == 50)
    {
      vga_output("Press enter to view next page or type quit to stop...\n");
      showingLog=1;
      return;
    }
  }
}

//log user configuration data///////////////////////////////////////////////////
void save_config()
{
  //set up write array
  unsigned char dataToSave[194] = "__ESNIFF_CFG_BEGIN__";
  
  //save logging, verbosity, capturing, status
  dataToSave[20]=logging;
  dataToSave[21]=verbosity;
  dataToSave[22]=capturing;
  dataToSave[23]=status_bar;
  
  //save filter - cast filter_Rules pointer
  unsigned int i=0;
  unsigned char * rules_ptr = (unsigned char *) filter_Rules;
  for(i=0;i<170;i++)
  {
    dataToSave[24+i]=(*(rules_ptr+i));
  }
  
  //tell user it's done
  if(!alt_flash_cfi_write(&fDev.dev,0,dataToSave,194))
  {
    vga_write_output(CHECKMARK);
    vga_output("User configuration data saved.\n\n"); 
  }
  else
  {
    vga_write_output(REDX);
    vga_output("ERROR writing user configuration to flash.\n\n"); 
  }
}

//restore user configuration data///////////////////////////////////////////////
void load_config()
{
  //read the data out of flash
  unsigned char tmp[194];
  if(alt_flash_cfi_read(&fDev.dev,0,tmp,194))
  {
    vga_write_output(REDX);
    vga_output("ERROR reading user configuration from flash.\n\n"); 
    return;
  }
  
  //if it begins with __ESNIFF_CFG_BEGIN__, the data should be valid
  if(strcmpbeg(tmp,"__ESNIFF_CFG_BEGIN__"))
  {
     //read in the global vars
     logging = tmp[20];
     verbosity = tmp[21];
     if(tmp[22] == 1)
     {
        alt_irq_enable(DM9000A_IRQ);
        capturing = 1;
     }
     else
     {
        alt_irq_disable(DM9000A_IRQ);
        capturing = 0;
     }
     status_bar = tmp[23];
     
     //read in the filter - cast filter rules to unsigned char type
     unsigned int i;
     unsigned char * rules_ptr = (unsigned char *) filter_Rules;
     for(i=0;i<170;i++)
     {
       (*(rules_ptr+i))=tmp[24+i];
     }
     
     //tell user it's all good in tha hood
     vga_write_output(CHECKMARK);
     vga_output("User configuration loaded.\n\n");
     return;
  }
  
  //otherwise, there was none saved
  vga_write_output(REDX);
  vga_output("No configuration data to load.\n\n");
}

#endif
